"""
Base client for Luna API backport platform 3
"""

from numbers import Real
from typing import Optional, Type

import aiohttp
from aiohttp import BasicAuth

from luna3.common.base_api import BaseHTTPAPI
from luna3.common.luna_request import guessContentType
from luna3.lunavl.mixins import BaseLunaHttpClient, MixinStub


class BaseLunaBP3HttpClient(BaseLunaHttpClient):
    """
    Luna API backport platform 3 client.

    Attributes:
        token (Optional[str]): auth token
        email (Optional[str]): email for auth
        password (Optional[str]): password for auth
        messageReceiveTimeout (Real): message receive timeout:
                                      time between start waiting message and message appearance
        _session: session to work in; is created automatically if 'asyncRequest'
    """

    def __init__(
        self,
        *,
        token: Optional[str] = None,
        email: Optional[str] = None,
        password: Optional[str] = None,
        messageReceiveTimeout: Optional[Real] = None,
        session: Optional[aiohttp.ClientSession] = None,
        **kwargs,
    ):
        super().__init__(**kwargs)
        self.token = token
        self.email = email
        self.password = password
        self.messageReceiveTimeout = messageReceiveTimeout
        self._session = session

    @staticmethod
    def getContentType(body: bytes) -> str:
        """
        Get mimetype, check it is supported.

        Args:
            body: body bytes

        Returns:
            image mimetype

        Raises:
            ValueError if mimetype is unsupported

        """
        availableTypes = (
            "image/jpeg",
            "image/png",
            "image/gif",
            "image/bmp",
            "image/tiff",
            "application/x-vl-xpk",
            "application/x-vl-face-descriptor",
            "image/x-portable-pixmap",
        )

        mimetype = guessContentType(body)
        if mimetype not in availableTypes:
            raise ValueError(f"Unsupported mimetype, availbale mimetypes are: {availableTypes}")
        return mimetype

    def getAuthHeader(  # pylint: disable=arguments-differ
        self, email: Optional[str] = None, password: Optional[str] = None, token: Optional[str] = None
    ) -> dict:
        """
        Get authorization header by token or login and password

        Args:
            email: email
            password: password
            token: token

        Returns:
            dict with Authorization header

        """
        token = self.token or token
        email = self.email or email
        password = self.password or password
        if token is not None and (email is not None and password is not None):
            raise ValueError("Authorization data does not set properly: required token or email and password pair")
        if token is not None:
            return {"X-Auth-Token": token}
        return {"Authorization": BasicAuth(login=email, password=password).encode()}

    def makeAuthRequest(
        self, login: Optional[str] = None, password: Optional[str] = None, token: Optional[str] = None, **requestKwargs
    ):
        """
        Prepare auth header and make request.

        Args:
            login: email (login)
            password: password
            token: token
            requestKwargs: standard request kwargs

        Returns:
            structure with status code, request and decoded Luna API response body is returned.

        """
        requestKwargs.setdefault("headers", {}).update(self.getAuthHeader(email=login, password=password, token=token))
        return self.makeRequest(**requestKwargs)

    makeRequest = BaseHTTPAPI.makeRequest


class Backport3MixinStub(MixinStub):
    """Backport3 stub."""

    # bp3 origin
    origin: str
    # get content type; real method is defined above (BaseLunaBP3HttpClient)
    getContentType: Type[BaseLunaBP3HttpClient.getContentType]
    # auth header; real method is defined above (BaseLunaBP3HttpClient)
    getAuthHeader: Type[BaseLunaBP3HttpClient.getAuthHeader]
    # make request with auth header; real method is defined above (BaseLunaBP3HttpClient)
    makeAuthRequest: Type[BaseLunaBP3HttpClient.makeAuthRequest]
    # make request with no auth header; real method is defined above (BaseLunaBP3HttpClient)
    makeRequest: Type[BaseHTTPAPI.makeRequest]
    # message receive timeout: time between start waiting message and message appearance
    messageReceiveTimeout: Real
