"""
Module realize linker mixin
"""

from typing import Awaitable, Union

from luna3.common.luna_response import LunaResponse

from .base import Backport3MixinStub


class LinkerMixin(Backport3MixinStub):
    """Linker Mixin to work with linker."""

    def linkDescriptorToPerson(
        self, personId: str, descriptorId: str, action: str, **kwargs
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        The function creates or deletes a link between a descriptor and a person.

        Args:
            personId: person id
            descriptorId: descriptor id
            action: "attach" or "detach"

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.

        """
        queries = {"descriptor_id": descriptorId, "do": action}
        return self.makeAuthRequest(
            url=f"{self.baseUri}/storage/persons/{personId}/linked_descriptors",
            method="PATCH",
            queryParams=queries,
            **kwargs,
        )

    def getLinkedDescriptorToPerson(self, personId: str, **kwargs) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        The function gets the list of descriptors, which are linked to a person.

        Args:
            personId: person id

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse` json with descriptors.

        """
        return self.makeAuthRequest(
            url=f"{self.baseUri}/storage/persons/{personId}/linked_descriptors", method="GET", **kwargs
        )

    def linkListToPerson(
        self, personId: str, listId: str, action: str, **kwargs
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        The function creates or deletes a link between a list and a person.

        Args:
            personId: person id
            listId: list id
            action: "attach" or "detach"

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.

        """
        queries = {"list_id": listId, "do": action}
        return self.makeAuthRequest(
            url=f"{self.baseUri}/storage/persons/{personId}/linked_lists", method="PATCH", queryParams=queries, **kwargs
        )

    def getLinkedListsToPerson(self, personId: str, **kwargs) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        The function gets the list of lists, which are linked to a person.

        Args:
            personId: person id

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse` json with lists.

        """
        return self.makeAuthRequest(
            url=f"{self.baseUri}/storage/persons/{personId}/linked_lists", method="GET", **kwargs
        )

    def linkListToDescriptor(
        self, descriptorId: str, listId: str, action: str, **kwargs
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        The function creates or deletes a link between a list and a descriptor.

        Args:
            descriptorId: descriptor id
            listId:  list id
            action: "attach" or "detach"

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.

        """
        queries = {"list_id": listId, "do": action}
        return self.makeAuthRequest(
            url=f"{self.baseUri}/storage/descriptors/{descriptorId}/linked_lists",
            method="PATCH",
            queryParams=queries,
            **kwargs,
        )

    def getLinkedListsToDescriptor(self, descriptorId: str, **kwargs) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        The function gets the list of lists, the descriptor is linked to.

        Args:
            descriptorId: descriptor id

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse` json with lists.

        """
        return self.makeAuthRequest(
            url=f"{self.baseUri}/storage/descriptors/{descriptorId}/linked_lists", method="GET", **kwargs
        )
