"""
Module realize matcher mixin
"""

from typing import Awaitable, List, Optional, Union

from luna3.common.luna_response import LunaResponse

from .base import Backport3MixinStub


class MatcherMixin(Backport3MixinStub):
    """Matcher Mixin to work with matcher."""

    @staticmethod
    def _setOneRequiredArgument(args: dict, defaultArgs: dict) -> type(None):
        """
        Sets one default parameter to "args" from default "t".

        Args:
            args: arguments to set default argument in
            defaultArgs: default arguments

        Raises:
            ValueError if no default peremeters were set

        """
        for params in defaultArgs:
            if defaultArgs[params] is not None:
                args[params] = defaultArgs[params]
                return
        raise ValueError

    def identify(
        self,
        personId: Optional[str] = None,
        descriptorId: Optional[str] = None,
        listId: Optional[str] = None,
        personIds: List[str] = None,
        limit: Optional[int] = 3,
        **kwargs,
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        The function matches a descriptor or person with a list of candidate persons.

        Either *descriptor_id* or *person_id* parameter should be specified as the reference and
        either *list_id* or *person_ids* parameter should be determined as the candidate.

        Args:
            personId: person id to take the reference descriptors from
            descriptorId: reference descriptor id.
            listId: candidate list id.
            personIds: list of candidate person ids
            limit: identify result limit

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.

        """
        queries = {"limit": limit}
        self._setOneRequiredArgument(queries, {"person_id": personId, "descriptor_id": descriptorId})
        self._setOneRequiredArgument(queries, {"list_id": listId, "person_ids": personIds})
        return self.makeAuthRequest(
            url=f"{self.baseUri}/matching/identify", method="POST", queryParams=queries, **kwargs
        )

    def match(
        self,
        personId: Optional[str] = None,
        descriptorId: Optional[str] = None,
        listId: Optional[str] = None,
        descriptorsIds: List[str] = None,
        limit: Optional[int] = 3,
        **kwargs,
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        The function matches a descriptor or a person with a list of candidate descriptors.

        Either *descriptor_id* or *person_id* parameter should be specified as the reference and either *list_id* or
        *descriptor_ids*  parameter should be determined as the candidate.

        Args:
            personId: person id to take the reference descriptors from
            descriptorId: reference descriptor id.
            listId: candidate list id.
            descriptorsIds: list of candidate descriptor ids
            limit: limit

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.

        """
        queries = {"limit": limit}
        self._setOneRequiredArgument(queries, {"person_id": personId, "descriptor_id": descriptorId})
        self._setOneRequiredArgument(queries, {"list_id": listId, "descriptor_ids": descriptorsIds})
        return self.makeAuthRequest(url=f"{self.baseUri}/matching/match", method="POST", queryParams=queries, **kwargs)

    def verify(self, descriptorId: str, personId: str, **kwargs) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        The function matches a descriptor with candidate person descriptors.

        Args:
            descriptorId: reference descriptor id.
            personId: reference person id.

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.

        """
        queries = {"person_id": personId, "descriptor_id": descriptorId}
        return self.makeAuthRequest(url=f"{self.baseUri}/matching/verify", method="POST", queryParams=queries, **kwargs)

    def search(
        self,
        body: Optional[bytes] = None,
        filename: Optional[str] = None,
        contentType: Optional[str] = None,
        limit: Optional[int] = 3,
        warpedImage: bool = False,
        estimateAttributes: bool = False,
        estimateEthnicities: bool = False,
        estimateEmotions: bool = False,
        estimateHeadPose: bool = False,
        estimateQuality: bool = False,
        scoreThreshold: float = 0,
        extractExif: bool = False,
        listId: str = None,
        pitchLt: Optional[float] = None,
        yawLt: Optional[float] = None,
        rollLt: Optional[float] = None,
        descriptorIds: Optional[List[str]] = None,
        personIds: Optional[List[str]] = None,
        noCache: bool = False,
        **kwargs,
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        The function extracts a descriptor from a photo, then matches it to a list of candidates.

        Either *list_id* or *person_ids* or *descriptor_ids* parameter should be specified as the candidate.

        Args:
            body: image bytes
            filename: path to folder with the image
            contentType: image mime type, if contentType is not set, will try to determine it by ourselves
                raise ValueError if mimetype of content is not matches with acceptable formats
                (Available mimetypes are: image/jpeg, image/png, image/gif, image/bmp, image/tiff,
                application/x-vl-xpk, application/x-vl-face-descriptor, image/x-portable-pixmap)
            warpedImage: Determines, whether an input image is a warped or an arbitrary one. Exact image warping
                algorithm is proprietary and this flag is intended for VisionLabs front-end tools only.

                The warped image has the following properties:

                * size is always 250x250 pixels;

                * color format is always RGB;

                * single face in a photo;

                * the face is always centered and rotated so that imaginary line between the eyes is horizontal.
            estimateAttributes: whether to estimate face attributes for the image or not (gender, age, glasses).
            estimateEthnicities: whether to estimate ethnicities from the image.
            estimateEmotions: Whether to estimate emotions from the image.
            estimateQuality: estimate face suitability for recognition
            scoreThreshold: If estimate_quality parameter is set to 1, it is possible to apply a threshold check
                to each estimation. All face detections with the quality below the threshold are
                ignored and no descriptors are extracted from them. The function proceeds as
                usual with all the remaining detections (if left).
            extractExif: Whether to extract EXIF meta information from the input image or not.

                Exact output varies since there are no mandatory data writing requirements both to the authoring
                software and digital cameras.

                This function only parses the tags and outputs their names and values as-is.
            estimateHeadPose: whether to estimate head pose from the image.
            listId: candidate list id.
            descriptorIds: list of candidate descriptor ids
            personIds: list of candidate person ids
            limit: search result limit
            yawLt: maximum deviation yaw angle from 0 (maximum 180)
            pitchLt: maximum deviation pitch angle from 0 (maximum 180)
            rollLt: maximum deviation roll angle from 0 (maximum 180)
            noCache: A boolean, whether to use cache or no

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.

        """
        if body is None and filename is None:
            raise ValueError

        if body is None:
            with open(filename, "rb") as imageFile:
                body = imageFile.read()

        if contentType is None:
            contentType = self.getContentType(body)
        headers = {"Content-Type": contentType}

        queries = {
            "estimate_attributes": int(estimateAttributes),
            "estimate_quality": int(estimateQuality),
            "warped_image": int(warpedImage),
            "extract_exif": int(extractExif),
            "limit": limit,
            "estimate_emotions": int(estimateEmotions),
            "estimate_ethnicities": int(estimateEthnicities),
            "estimate_head_pose": int(estimateHeadPose),
            "score_threshold": scoreThreshold,
            "pitch__lt": pitchLt,
            "yaw__lt": yawLt,
            "roll__lt": rollLt,
            "no_cache": int(noCache),
        }

        self._setOneRequiredArgument(
            queries, {"list_id": listId, "person_ids": personIds, "descriptor_ids": descriptorIds}
        )

        return self.makeAuthRequest(
            url=f"{self.baseUri}/matching/search",
            method="POST",
            queryParams=queries,
            body=body,
            headers=headers,
            **kwargs,
        )
